@@ -22,12 +22,13 @@ if on_rtd():
     jit = _fake_decorator
     njit = _fake_decorator
     stencil = _fake_decorator
+
     overload = _fake_decorator
     register_jitable = _fake_decorator
-
+    intrinsic = _fake_decorator
 else:
     from numba import cfunc, jit, njit, generated_jit, stencil  # noqa
-    from numba.extending import overload, register_jitable  # noqa
+    from numba.extending import overload, register_jitable, intrinsic  # noqa
 
 
 def is_numba_type_none(arg):
